import javafx.application.Application;
import javafx.event.EventHandler;
import javafx.scene.Group;
import javafx.scene.PerspectiveCamera;
import javafx.scene.Scene;
import javafx.scene.input.KeyCode;
import javafx.scene.input.KeyEvent;
import javafx.scene.transform.Rotate;
import javafx.scene.transform.Translate;
import javafx.stage.Stage;
import object.movableObject.Puck;
import object.staticObject.Paddle;
import object.staticObject.Wall;

public class Main extends Application implements Wall.GameStopper, EventHandler<KeyEvent> {
	
	private static final double WIDTH            = 800;
	private static final double HEIGHT           = 800;
	private static final double NEAR_CLIP        = 0.1;
	private static final double FAR_CLIP         = 5000;
	private static final double CAMERA_Z         = -2500;
	private static final double PUCK_RADIUS      = 20;
	private static final double PUCK_HEIGHT      = 20;
	private static final double LONG_WALL_WIDTH  = 1000;
	private static final double SHORT_WALL_WIDTH = 500;
	private static final double WALL_HEIGHT      = PUCK_HEIGHT;
	private static final double WALL_DEPTH       = 20;
	private static final double PADDLE_WIDTH     = 100;
	private static final double PADDLE_HEIGHT    = 20;
	private static final double PADDLE_DEPTH     = 20;
	private static final double STEP             = 10;
	private              Timer  timer;
	private              Paddle leftPaddle, rightPaddle;
	private PerspectiveCamera camera;
	
	public static void main ( String[] arguments ) {
		launch ( arguments );
	}
	
	private void createCameras ( ) {
		this.camera = new PerspectiveCamera ( true );
		this.camera.setNearClip ( NEAR_CLIP );
		this.camera.setFarClip ( FAR_CLIP );
		this.camera.getTransforms ( ).addAll (
				new Rotate ( -45, Rotate.X_AXIS ),
				new Translate ( 0, 0, CAMERA_Z )
		);
	}
	
	private Scene getScene ( ) {
		Group root = new Group ( );
		
		Scene scene = new Scene ( root, WIDTH, HEIGHT, true );
		
		Puck puck = new Puck ( PUCK_RADIUS, PUCK_HEIGHT );
		root.getChildren ( ).addAll ( puck );
		
		Wall upperWall = new Wall ( LONG_WALL_WIDTH, WALL_HEIGHT, WALL_DEPTH, Wall.Type.UPPER );
		Wall lowerWall = new Wall ( LONG_WALL_WIDTH, WALL_HEIGHT, WALL_DEPTH, Wall.Type.LOWER );
		Wall leftWall  = new Wall ( SHORT_WALL_WIDTH, WALL_HEIGHT, WALL_DEPTH, Wall.Type.LEFT, this );
		Wall rightWall = new Wall ( SHORT_WALL_WIDTH, WALL_HEIGHT, WALL_DEPTH, Wall.Type.RIGHT, this );
		
		upperWall.getTransforms ( ).addAll (
				new Translate ( 0, 0, SHORT_WALL_WIDTH / 2 + WALL_DEPTH / 2 )
		);
		lowerWall.getTransforms ( ).addAll (
				new Translate ( 0, 0, -( SHORT_WALL_WIDTH / 2 + WALL_DEPTH / 2 ) )
		);
		leftWall.getTransforms ( ).addAll (
				new Translate ( -( LONG_WALL_WIDTH / 2 - WALL_DEPTH / 2 ), 0, 0 ),
				new Rotate ( 90, Rotate.Y_AXIS )
		);
		rightWall.getTransforms ( ).addAll (
				new Translate ( LONG_WALL_WIDTH / 2 - WALL_DEPTH / 2, 0, 0 ),
				new Rotate ( 90, Rotate.Y_AXIS )
		);
		
		root.getChildren ( ).addAll ( upperWall, lowerWall, leftWall, rightWall );
		
		this.leftPaddle = new Paddle ( PADDLE_WIDTH, PADDLE_HEIGHT, PADDLE_DEPTH );
		this.rightPaddle = new Paddle ( PADDLE_WIDTH, PADDLE_HEIGHT, PADDLE_DEPTH );
		this.leftPaddle.getTransforms ( ).addAll (
				new Translate ( -LONG_WALL_WIDTH / 2 * 0.8, 0, 0 ),
				new Rotate ( 90, Rotate.Y_AXIS )
		);
		this.rightPaddle.getTransforms ( ).addAll (
				new Translate ( LONG_WALL_WIDTH / 2 * 0.8, 0, 0 ),
				new Rotate ( 90, Rotate.Y_AXIS )
		);
		
		root.getChildren ( ).addAll ( this.leftPaddle, this.rightPaddle );
		
		this.createCameras ( );
		scene.setCamera ( this.camera );
		
		this.timer = new Timer ( puck, upperWall, lowerWall, leftWall, rightWall, this.rightPaddle, this.leftPaddle );
		this.timer.start ( );
		
		scene.addEventHandler ( KeyEvent.KEY_PRESSED, this );
		
		return scene;
	}
	
	@Override public void start ( Stage primaryStage ) throws Exception {
		primaryStage.setScene ( this.getScene ( ) );
		primaryStage.setTitle ( "Vazdusni hokej" );
		primaryStage.show ( );
	}
	
	@Override public void stopGame ( ) {
		this.timer.stop ( );
	}
	
	@Override public void handle ( KeyEvent event ) {
		if ( event.getCode ( ).equals ( KeyCode.W ) ) {
			double newZ = this.leftPaddle.getZ ( ) + STEP;
			
			if ( ( newZ + PADDLE_WIDTH / 2 ) <= SHORT_WALL_WIDTH / 2 ) {
				this.leftPaddle.move ( 0, 0, STEP );
			}
		} else if ( event.getCode ( ).equals ( KeyCode.S ) ) {
			double newZ = this.leftPaddle.getZ ( ) - STEP;
			
			if ( ( newZ - PADDLE_WIDTH / 2 ) >= -SHORT_WALL_WIDTH / 2 ) {
				this.leftPaddle.move ( 0, 0, -STEP );
			}
		} else if ( event.getCode ( ).equals ( KeyCode.UP ) ) {
			double newZ = this.rightPaddle.getZ ( ) + STEP;
			
			if ( ( newZ + PADDLE_WIDTH / 2 ) <= SHORT_WALL_WIDTH / 2 ) {
				this.rightPaddle.move ( 0, 0, STEP );
			}
		} else if ( event.getCode ( ).equals ( KeyCode.DOWN ) ) {
			double newZ = this.rightPaddle.getZ ( ) - STEP;
			
			if ( ( newZ - PADDLE_WIDTH / 2 ) >= -SHORT_WALL_WIDTH / 2 ) {
				this.rightPaddle.move ( 0, 0, -STEP );
			}
		}
	}
}
