package object.staticObject;

import javafx.scene.paint.Color;
import javafx.scene.paint.PhongMaterial;
import javafx.scene.shape.Box;
import object.movableObject.MovableObject;

public class Wall extends StaticObject {
	private Type        type;
	private GameStopper gameStopper;
	
	public Wall ( double width, double height, double depth, Type type, GameStopper gameStopper ) {
		Box wall = new Box ( width, height, depth );
		wall.setMaterial ( new PhongMaterial ( Color.BROWN ) );
		super.getChildren ( ).addAll ( wall );
		
		this.type = type;
		this.gameStopper = gameStopper;
	}
	
	public Wall ( double width, double height, double depth, Type type ) {
		this ( width, height, depth, type, null );
	}
	
	@Override public boolean collision ( MovableObject movableObject ) {
		boolean collisionDetected = super.collision ( movableObject );
		
		if ( collisionDetected ) {
			switch ( type ) {
				case UPPER:
				case LOWER: {
					movableObject.invertSpeedZ ( );
					break;
				}
				case LEFT:
				case RIGHT: {
					if ( this.gameStopper != null ) {
						this.gameStopper.stopGame ( );
					}
					break;
				}
			}
		}
		
		return collisionDetected;
	}
	
	public enum Type {
		UPPER, LOWER, LEFT, RIGHT
	}
	
	
	public static interface GameStopper {
		void stopGame ( );
	}
}